import gym
from gym import spaces
from gym.utils import seeding
import numpy as np

class LqrEnv(gym.Env):

    def __init__(self, size, init_state, state_bound):
        self.init_state = init_state
        self.size = size 
        self.action_space = spaces.Box(low=-state_bound, high=state_bound, shape=(size,))
        self.observation_space = spaces.Box(low=-state_bound, high=state_bound, shape=(size,))
        self._seed()

    def _seed(self, seed=None):
        self.np_random, seed = seeding.np_random(seed)
        return [seed]

    def _step(self,u):
        costs = np.sum(u**2) + np.sum(self.state**2)
        self.state = np.clip(self.state + u, self.observation_space.low, self.observation_space.high)
        return self._get_obs(), -costs, False, {}

    def _reset(self):
        high = self.init_state*np.ones((self.size,))
        self.state = self.np_random.uniform(low=-high, high=high)
        self.last_u = None
        return self._get_obs()

    def _get_obs(self):
        return self.state
    
if __name__ == '__main__':
    from gym.envs.registration import register

    register(
        id='MyEnv-v0',
        entry_point='myenv.myenv:MyEnv',
    )


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import sasrl_env.common.python.envp_pb2 as envp__pb2


class EnvpStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Makep = channel.unary_unary(
                '/Envp/Makep',
                request_serializer=envp__pb2.Namep.SerializeToString,
                response_deserializer=envp__pb2.Infop.FromString,
                )
        self.Resetp = channel.unary_unary(
                '/Envp/Resetp',
                request_serializer=envp__pb2.EnvIDsp.SerializeToString,
                response_deserializer=envp__pb2.Observationsp.FromString,
                )
        self.Renderp = channel.unary_unary(
                '/Envp/Renderp',
                request_serializer=envp__pb2.RenderModesp.SerializeToString,
                response_deserializer=envp__pb2.RenderOutsp.FromString,
                )
        self.Seedp = channel.unary_unary(
                '/Envp/Seedp',
                request_serializer=envp__pb2.EnvSeedsp.SerializeToString,
                response_deserializer=envp__pb2.Emptyp.FromString,
                )
        self.Stepp = channel.unary_unary(
                '/Envp/Stepp',
                request_serializer=envp__pb2.Actionsp.SerializeToString,
                response_deserializer=envp__pb2.Transitionsp.FromString,
                )
        self.Samplep = channel.unary_unary(
                '/Envp/Samplep',
                request_serializer=envp__pb2.Emptyp.SerializeToString,
                response_deserializer=envp__pb2.Actionsp.FromString,
                )
        self.Closep = channel.unary_unary(
                '/Envp/Closep',
                request_serializer=envp__pb2.EnvIDsp.SerializeToString,
                response_deserializer=envp__pb2.Emptyp.FromString,
                )


class EnvpServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Makep(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Resetp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Renderp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Seedp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Stepp(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Samplep(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Closep(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_EnvpServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Makep': grpc.unary_unary_rpc_method_handler(
                    servicer.Makep,
                    request_deserializer=envp__pb2.Namep.FromString,
                    response_serializer=envp__pb2.Infop.SerializeToString,
            ),
            'Resetp': grpc.unary_unary_rpc_method_handler(
                    servicer.Resetp,
                    request_deserializer=envp__pb2.EnvIDsp.FromString,
                    response_serializer=envp__pb2.Observationsp.SerializeToString,
            ),
            'Renderp': grpc.unary_unary_rpc_method_handler(
                    servicer.Renderp,
                    request_deserializer=envp__pb2.RenderModesp.FromString,
                    response_serializer=envp__pb2.RenderOutsp.SerializeToString,
            ),
            'Seedp': grpc.unary_unary_rpc_method_handler(
                    servicer.Seedp,
                    request_deserializer=envp__pb2.EnvSeedsp.FromString,
                    response_serializer=envp__pb2.Emptyp.SerializeToString,
            ),
            'Stepp': grpc.unary_unary_rpc_method_handler(
                    servicer.Stepp,
                    request_deserializer=envp__pb2.Actionsp.FromString,
                    response_serializer=envp__pb2.Transitionsp.SerializeToString,
            ),
            'Samplep': grpc.unary_unary_rpc_method_handler(
                    servicer.Samplep,
                    request_deserializer=envp__pb2.Emptyp.FromString,
                    response_serializer=envp__pb2.Actionsp.SerializeToString,
            ),
            'Closep': grpc.unary_unary_rpc_method_handler(
                    servicer.Closep,
                    request_deserializer=envp__pb2.EnvIDsp.FromString,
                    response_serializer=envp__pb2.Emptyp.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Envp', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Envp(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Makep(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Envp/Makep',
            envp__pb2.Namep.SerializeToString,
            envp__pb2.Infop.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Resetp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Envp/Resetp',
            envp__pb2.EnvIDsp.SerializeToString,
            envp__pb2.Observationsp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Renderp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Envp/Renderp',
            envp__pb2.RenderModesp.SerializeToString,
            envp__pb2.RenderOutsp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Seedp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Envp/Seedp',
            envp__pb2.EnvSeedsp.SerializeToString,
            envp__pb2.Emptyp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Stepp(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Envp/Stepp',
            envp__pb2.Actionsp.SerializeToString,
            envp__pb2.Transitionsp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Samplep(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Envp/Samplep',
            envp__pb2.Emptyp.SerializeToString,
            envp__pb2.Actionsp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Closep(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Envp/Closep',
            envp__pb2.EnvIDsp.SerializeToString,
            envp__pb2.Emptyp.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import numpy as np
import operator
from multiprocessing import Process, Pipe
from .DictEnv import EnvStatus, DictEnv, CloudpickleWrapper


def worker(remote, parent_remote, env_fn_wrapper, env_name):
    parent_remote.close()
    env = env_fn_wrapper.x(env_name)
    while True:
        cmd, data = remote.recv()
        if cmd == 'step':
            ob, reward, done, info = env.step(data)
            if done:
                ob = env.reset()
            remote.send((ob, reward, done, info))
        elif cmd == 'reset':
            ob = env.reset()
            remote.send(ob)
        elif cmd == 'seed':
            seed = env.seed(data)
            remote.send(seed)
        elif cmd == 'render':
            render_out = env.render(mode=data)
            remote.send(render_out)
        elif cmd == 'close':
            remote.close()
            break
        elif cmd == 'get_spaces':
            remote.send((env.observation_space, env.action_space))
        elif cmd == 'get_max_episode_steps':
            remote.send(env._max_episode_steps)
        else:
            raise NotImplementedError


class SubprocDictEnv(DictEnv):
    def __init__(self, observation_space=None, action_space=None):
        """
        envs: list of gym environments to run in subprocesses
        """
        self.status = {}
        self.waiting = False
        self.closed = False
        self.remotes, self.work_remotes = {}, {}
        self.ps = {}
        self.master_env_id = None
        self.observation_space = {}
        self.action_space = {}
        self._max_episode_steps = {}

        # check if observation_space and action_space are passed manually
        DictEnv.__init__(self)

    def add_env(self, env_fn, env_name):

        # get key for env instance
        try:
            env_id = str(max(map(int, self.status.keys())) + 1)
            assert env_id != self.master_env_id
        except ValueError:
            env_id = str(0)

        # place env in master if there is no other env
        if self.master_env_id == None:
            self.ps[env_id] = 'master'
            self.status[env_id] = EnvStatus.IDLE
            self.master_env_id = env_id

            # create env
            env = env_fn(env_name)
            self.observation_space[env_id] = env.observation_space
            self.action_space[env_id] = env.action_space
            self._max_episode_steps[env_id] = env._max_episode_steps
            self.remotes[env_id] = env

        else:  # start a new process and create an instance there
            # start a process
            remote, work_remote = Pipe()
            p = Process(target=worker, args=(work_remote, remote, CloudpickleWrapper(env_fn), env_name))

            self.remotes[env_id] = remote
            self.work_remotes[env_id] = work_remote
            self.ps[env_id] = p
            self.status[env_id] = EnvStatus.IDLE

            # start
            p.start()
            work_remote.close()

            # read observation_space, action_space and max_episode_steps
            self.remotes[env_id].send(('get_spaces', None))
            observation_space, action_space = self.remotes[env_id].recv()
            self.observation_space[env_id] = observation_space
            self.action_space[env_id] = action_space
            self.remotes[env_id].send(('get_max_episode_steps', None))
            max_episode_steps = self.remotes[env_id].recv()
            self._max_episode_steps[env_id] = max_episode_steps

        # increment self.num_envs
        self.inc_num(1)
        return env_id

    def step(self, actions):
        master_i = -1
        # for efficiency, send calls to remotes first and then run the master job
        for i in range(len(actions.env_id)):
            env_id = actions.env_id[i]
            if env_id != self.master_env_id:
                action = actions.data[i]
                self.remotes[env_id].send(('step', action))
                self.status[env_id] = EnvStatus.WAITING
            else:
                master_i = i

        # run the env on master
        if master_i != -1:
            env_id = actions.env_id[master_i]
            action = actions.data[master_i]
            master_result = self.remotes[env_id].step(action)
            self.status[env_id] = EnvStatus.IDLE

        # gather results
        results = []
        for i in range(len(actions.env_id)):
            env_id = actions.env_id[i]
            if env_id != self.master_env_id:
                results.append(self.remotes[env_id].recv())
                self.status[env_id] = EnvStatus.IDLE
            else:
                results.append(master_result)

        obs, rews, dones, infos = zip(*results)
        return obs, rews, dones, infos

    def reset(self, env_ids):
        reset_master = False
        for env_id in env_ids:
            if env_id != self.master_env_id:
                self.remotes[env_id].send(('reset', None))
            else:
                reset_master = True
        if reset_master:
            master_obs = self.remotes[self.master_env_id].reset()

        # gather observations
        results = []
        for env_id in env_ids:
            if env_id != self.master_env_id:
                results.append(self.remotes[env_id].recv())
            else:
                results.append(master_obs)
        return results

    def render(self, render_msgs):
        master_i = -1

        for i in range(len(render_msgs.env_id)):
            env_id = render_msgs.env_id[i]
            if env_id != self.master_env_id:
                mode = render_msgs.data[i]
                self.remotes[env_id].send(('render', mode))
                self.status[env_id] = EnvStatus.WAITING
            else:
                master_i = i

        # run the env on master
        if master_i != -1:
            env_id = render_msgs.env_id[master_i]
            mode = render_msgs.data[master_i]
            master_result = self.remotes[env_id].render(mode=mode)
            self.status[env_id] = EnvStatus.IDLE

        # gather results to make sure the render is finished on remotes
        results = []
        for i in range(len(render_msgs.env_id)):
            env_id = render_msgs.env_id[i]
            if env_id != self.master_env_id:
                results.append(self.remotes[env_id].recv())
                self.status[env_id] = EnvStatus.IDLE
            else:
                results.append(master_result)
        return results

    def seed(self, seeds_msgs):

        for i in range(len(seeds_msgs.env_id)):
            env_id = seeds_msgs.env_id[i]
            if env_id != self.master_env_id:
                seed = seeds_msgs.data[i]
                self.remotes[env_id].send(('seed', seed))
                self.status[env_id] = EnvStatus.WAITING
            else:
                env_id = seeds_msgs.env_id[i]
                seed = seeds_msgs.data[i]
                self.remotes[env_id].seed(seed)
                self.status[env_id] = EnvStatus.IDLE

        # gather results to make sure the seed is finished on remotes
        for i in range(len(seeds_msgs.env_id)):
            env_id = seeds_msgs.env_id[i]
            if env_id != self.master_env_id:
                self.remotes[env_id].recv()
                self.status[env_id] = EnvStatus.IDLE

    def close(self, env_ids):
        for env_id in env_ids:
            if self.status[env_id] == EnvStatus.WAITING and env_id != self.master_env_id:
                self.remotes[env_id].recv()

            if env_id != self.master_env_id:
                self.remotes[env_id].send(('close', None))
                self.ps[env_id].join()
            else:
                self.remotes[env_id].close()
                self.master_env_id = None
            del self.remotes[env_id]
            del self.ps[env_id]
            del self.status[env_id]



from concurrent import futures
import socket
import argparse
import grpc

# get installed package starting with gym prefix
import pkg_resources

from sasrl_env.common.python.env_pb2_grpc import EnvServicer as Service, \
    add_EnvServicer_to_server as register
from sasrl_env.serverSingle import Env
from sasrl_env.common.python.envp_pb2_grpc import EnvpServicer as Servicep, \
    add_EnvpServicer_to_server as registerp
from sasrl_env.serverMulti import Envp

installed_packages = pkg_resources.working_set
installed_packages_list = sorted(["%s==%s" % (i.key, i.version)
                                  for i in installed_packages
                                  if (i.key.startswith('gym') or i.key.endswith('gym'))])

from sasrl_env.common.python.utils import get_logger
logger = get_logger(log_level='info')
logger.info("Available gym packages: " + str(installed_packages_list))

# gym packages to import
import gym

modules = sorted(["%s" % i.key.replace('-', '_')
                  for i in installed_packages
                  if ((i.key.startswith('gym') or i.key.endswith('gym')) and (i.key != 'gym'))])
for library in modules:
    try:
        exec("import {module}".format(module=library))
    except Exception as e:
        print(e)


def get_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        s.connect(('10.255.255.255', 1))
        IP = s.getsockname()[0]
    except Exception:
        IP = '127.0.0.1'
    finally:
        s.close()
    return IP


class register_server():
    def start(self, port, mode='single', max_threads=8):
        host = get_ip()
        address = '{}:{}'.format(host, port)
        if mode == 'single':
            server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
            register(Env(), server)
        elif mode == 'multi':
            server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_threads))
            registerp(Envp(), server)

        server.add_insecure_port(address)
        server.start()
        logger.info("started server at: {}".format(address))
        server.wait_for_termination()
        return 0

def start(port, mode='single'):
    rs = register_server()
    rs.start(port, mode)
    return 0


if __name__ == '__main__':
    parser = argparse.ArgumentParser('environment server')
    parser.add_argument('--port', type=int, default=10005)
    parser.add_argument('--mode', type=str, default="single")
    args = parser.parse_args()

    start(args.port, args.mode)



import grpc
import numpy as np
import time

from sasrl_env.common.python.envp_pb2 import Actionsp, Namep, EnvIDsp, Emptyp

from sasrl_env.common.python.envp_pb2_grpc import EnvpStub


def decode_observation(observation):
    if not observation.data:
        return
    return np.asarray(observation.data).reshape(observation.shape)


class Object:
    pass


class Env:

    def __init__(self, address):
        self.channel = grpc.insecure_channel(address)
        self.env_stub = EnvpStub(self.channel)
        self.envs = {}

    def make(self, name):
        info = self.env_stub.Make(Namep(data=name))
        env_id = info.env_id
        self.envs[env_id] = Object()
        self.envs[env_id].observation_space = Object()
        self.envs[env_id].action_space = Object()
        self.envs[env_id].observation_space.shape = tuple(info.observation_shape)
        self.envs[env_id].action_space.n = info.num_actions
        self.envs[env_id]._max_episode_steps = info.max_episode_steps
        self.envs[env_id].env_id = env_id
        return env_id

    def reset(self, env_ids=None):
        if env_ids == None:
            env_ids = self.envs.keys()
        env_ids = EnvIDsp(data=env_ids)
        observations = self.env_stub.Reset(env_ids)
        obs_lst = []
        for obs in observations.obs:
            obs_lst.append(decode_observation(obs))
        return obs_lst

    def step(self, action):

        env_id = []
        actions = []
        for k, v in action.items():
            env_id.append(k)
            actions.append(v)

        transition = self.env_stub.Step(Actionsp(env_id=env_id, data=actions))

        next_observations, rewards, dones = [], [], []
        for trans in transition.trans:
            next_observations.append(decode_observation(trans.next_observation))
            rewards.append(trans.reward)
            dones.append(trans.done)
        return next_observations, rewards, dones

    def Sample(self):
        action = self.env_stub.Sample(Emptyp())
        return action

    def close(self, env_ids=None):
        if env_ids == None:
            env_ids = self.envs.keys
        self.env_stub.Close(EnvIDsp(data=env_ids))


if __name__ == '__main__':
    host = '10.122.32.31'
    port = '10005'
    address = '{}:{}'.format(host, port)

    env = Env(address)
    # env.make('CartPole-v0')
    # env.make('Pong-v0')
    env_id = env.make('CartPole-v0')

    st = time.time()
    cnt = 0
    for i in range(500):
        s = env.reset(env_ids = [env_id])
        cnt += 1
        action = {env_id: 1}
        done = [False]
        while not done[0]:
            ns, r, done = env.step(action)
            cnt += 1
            s = ns
            # report
            if cnt % 100 == 0:
                print("cnt: {} -- rate: {}".format(cnt,cnt/(time.time()-st)))
    env.close([env_id])

    print('done')

    # cnt = 0
    # for i in range(100):
    #     s = env.reset()
    #     cnt += 1
    #     done = False
    #     while not done:
    #         ns, r, done = env.step(1)
    #         cnt += 1
    #         s = ns
    #         # report
    #         if cnt % 100 == 0:
    #             print("cnt: {} -- rate: {}".format(cnt,cnt/(time.time()-st)))
    # env1.close()



from concurrent import futures
import argparse
import warnings
# get installed package starting with gym prefix
import pkg_resources
import grpc
from sasrl_env.env_dict.SubProcDict import SubprocDictEnv
import os

from sasrl_env.common.python.envp_pb2 import Infop, Observationp, Observationsp, Transitionp, Transitionsp, \
    Actionsp, Emptyp, RenderOutsp, RGBArryp
from sasrl_env.common.python.envp_pb2_grpc import EnvpServicer as Servicep, add_EnvpServicer_to_server as registerp

import gym

from sasrl_env.common.python.utils import get_logger
logger = get_logger(log_level='info')


def make_env(env_name):
    env = gym.make(env_name)
    return env


def encode_observation(observation):
    return Observationp(data=observation.ravel(), shape=observation.shape)


class Envp(Servicep):
    env = SubprocDictEnv()

    def Makep(self, name, _):
        name = name.data
        env_id = self.env.add_env(make_env, name)
        logger.info('env {} with id {} created'.format(name, env_id))

        # check validity of observation_space
        try:
            self.env.observation_space[env_id]
        except:
            raise NameError('Environment should have an observation_space object. Use either Box or Discrete to define the space object.')

        try:
            if self.env.observation_space[env_id].__class__ == gym.spaces.box.Box:
                observation_space_type = 'box'
            elif self.env.observation_space[env_id].__class__ == gym.spaces.Discrete:
                observation_space_type = 'discrete'
        except:
            observation_space_type = 'none'

        # check validity of action_space
        try:
            self.env.action_space[env_id]
        except:
            raise NameError('Environment should have an action_space object of type Discrete.')

        if self.env.action_space[env_id].__class__ != gym.spaces.Discrete:
            action_space_type_ = "continues"
            raise NotImplementedError('Discrete is the only supported space for action_space.')
        else:
            num_actions = self.env.action_space[env_id].n
            action_space_type_ = "discrete"

        return Infop(observation_space_type=observation_space_type,
                     observation_shape=self.env.observation_space[env_id].shape,
                     num_actions=self.env.action_space[env_id].n,
                     max_episode_steps=self.env._max_episode_steps[env_id],
                     action_space_type=action_space_type_,
                     env_id=env_id)

    def Resetp(self, env_ids, _):
        obs = self.env.reset(env_ids.data)
        env_obs = [encode_observation(ob) for ob in obs]
        obs = Observationsp(obs=env_obs)
        return obs

    def Stepp(self, actions, _):

        next_observations, rewards, dones, _ = self.env.step(actions)
        transitions = []
        for i in range(len(next_observations)):
            next_obs = encode_observation(next_observations[i])
            transitions.append(Transitionp(next_observation=next_obs,
                                           reward=rewards[i],
                                           done=dones[i]))
        return Transitionsp(trans=transitions)

    def Samplep(self, env_ids, _):
        # todo: test this function
        action = self.env.action_space.sample()
        return Actionsp(data=action)

    def Renderp(self, renders, _):
        res = self.env.render(renders)

        mode = renders.data
        if mode == 'rgb_array':
            reno = RenderOutsp(rgb_array=[RGBArryp(rgb=r.flatten()) for r in res])
        elif mode == 'ansi':
            reno = RenderOutsp(ansi=res)
        elif mode == 'human':
            reno = RenderOutsp()
        else:
            raise Exception("render mode {} not supported.".format(mode))

        return reno

    def Seedp(self, seeds, _):
        if hasattr(self.env, 'seed'):
            self.env.seed(seeds)
        else:
            logger.warning("There is no function to set seed in the environment.")
        return Emptyp()

    def Closep(self, env_ids, _):
        self.env.close(env_ids.data)
        return Emptyp()


if __name__ == '__main__':
    parser = argparse.ArgumentParser('environment server')
    parser.add_argument('--host', type=str)
    parser.add_argument('--port', type=str)
    parser.add_argument('--get_pid', type=bool, default=True)

    args = parser.parse_args()

    if args.get_pid:
        pid = os.getpid()
        file1 = open(".pid.txt", "w")
        file1.writelines(str(pid))
        file1.close()

    address = '{}:{}'.format(args.host, args.port)

    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
    registerp(Envp(), server)
    server.add_insecure_port(address)
    server.start()
    server.wait_for_termination()



import logging
import os
from pathlib import Path
home = str(Path.home())

def get_logger(log_dir=home+'/.SASGym',log_level='debug'):
    # set logger
    logger = logging.getLogger()
    logFormatter = logging.Formatter("%(asctime)s  %(name)s  %(levelname)s  [%(funcName)s:%(lineno)d]  %(message)s")
    # logFormatter = logging.Formatter("%(asctime)s [%(levelname)-5.5s]  %(message)s")

    # create cache folder if it does not exist
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

    # create file handler for logger
    fileHandler = logging.FileHandler("{0}/{1}.log".format(log_dir, 'hist'))
    fileHandler.setFormatter(logFormatter)
    logger.addHandler(fileHandler)

    # create stream handler for logger
    consoleHandler = logging.StreamHandler()
    consoleHandler.setFormatter(logFormatter)
    logger.addHandler(consoleHandler)

    if log_level == 'info':
        logger.setLevel(logging.INFO)
    elif log_level == 'debug':
        logger.setLevel(logging.DEBUG)
    elif log_level == 'warning':
        logger.setLevel(logging.WARNING)
    elif log_level == 'error':
        logger.setLevel(logging.ERROR)

    return logger


import grpc
import numpy as np

from sasrl_env.common.python.env_pb2 import Action, Empty, Name
from sasrl_env.common.python.env_pb2_grpc import EnvStub

def decode_observation(observation):
    if not observation.data:
        return
    return np.asarray(observation.data).reshape(observation.shape)

class Object:
    pass

class Env:

    def __init__(self, name, address):
        self.channel = grpc.insecure_channel(address)
        self.env = EnvStub(self.channel)
        self.make(name)

    def make(self, name):
        info = self.env.Make(Name(data=name))
        self.observation_space = Object()
        self.action_space = Object()
        self.observation_space.shape = tuple(info.observation_shape)
        self.action_space.n = info.num_actions
        self._max_episode_steps = info.max_episode_steps

    def reset(self):
        return decode_observation(self.env.Reset(Empty()))

    def step(self, action):
        transition = self.env.Step(Action(data=action))
        next_observation = decode_observation(transition.next_observation)
        return next_observation, transition.reward, transition.done

    def Sample(self):
        action = self.env.Sample(Empty())
        return action
    def Close(self):
        self.env.Close()

    def close(self):
        self.channel.close()

if __name__ == '__main__':
    import time
    host = '10.122.32.31'
    port = '13001'
    address = '{}:{}'.format(host, port)

    # env = Env('Pong-v0', address)
    env = Env('CartPole-v0', address)

    st = time.time()
    cnt = 0
    for i in range(1000):
        s = env.reset()
        cnt += 1
        done = False
        while not done:
            ns, r, done = env.step(1)
            cnt += 1
            s = ns
            # report
            if cnt % 100 == 0:
                print("cnt: {} -- rate: {}".format(cnt,cnt/(time.time()-st)))
    env.close()


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: envp.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='envp.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\nenvp.proto\"\x15\n\x05Namep\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\"\xdc\x01\n\x05Infop\x12\x1e\n\x16observation_space_type\x18\x01 \x01(\t\x12\x19\n\x11observation_shape\x18\x02 \x03(\x05\x12\x1d\n\x15observation_space_low\x18\x03 \x03(\x02\x12\x1e\n\x16observation_space_high\x18\x04 \x03(\x02\x12\x13\n\x0bnum_actions\x18\x05 \x01(\x05\x12\x19\n\x11max_episode_steps\x18\x06 \x01(\x05\x12\x19\n\x11\x61\x63tion_space_type\x18\x07 \x01(\t\x12\x0e\n\x06\x65nv_id\x18\x08 \x01(\t\"\x08\n\x06\x45mptyp\"+\n\x0cObservationp\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x02\x12\r\n\x05shape\x18\x02 \x03(\x05\"+\n\rObservationsp\x12\x1a\n\x03obs\x18\x01 \x03(\x0b\x32\r.Observationp\"(\n\x08\x41\x63tionsp\x12\x0e\n\x06\x65nv_id\x18\x01 \x03(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x03(\x05\"\x17\n\x07\x45nvIDsp\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\t\"T\n\x0bTransitionp\x12\'\n\x10next_observation\x18\x01 \x01(\x0b\x32\r.Observationp\x12\x0e\n\x06reward\x18\x02 \x01(\x02\x12\x0c\n\x04\x64one\x18\x03 \x01(\x08\"+\n\x0cTransitionsp\x12\x1b\n\x05trans\x18\x01 \x03(\x0b\x32\x0c.Transitionp\",\n\x0cRenderModesp\x12\x0e\n\x06\x65nv_id\x18\x01 \x03(\t\x12\x0c\n\x04mode\x18\x02 \x03(\t\"3\n\x0bRenderOutsp\x12\x16\n\x03rgb\x18\x01 \x03(\x0b\x32\t.RGBArryp\x12\x0c\n\x04\x61nsi\x18\x02 \x03(\t\"\x18\n\x08RGBArryp\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x05\")\n\tEnvSeedsp\x12\x0e\n\x06\x65nv_id\x18\x01 \x03(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x03(\x05\x32\xf6\x01\n\x04\x45nvp\x12\x19\n\x05Makep\x12\x06.Namep\x1a\x06.Infop\"\x00\x12$\n\x06Resetp\x12\x08.EnvIDsp\x1a\x0e.Observationsp\"\x00\x12(\n\x07Renderp\x12\r.RenderModesp\x1a\x0c.RenderOutsp\"\x00\x12\x1e\n\x05Seedp\x12\n.EnvSeedsp\x1a\x07.Emptyp\"\x00\x12#\n\x05Stepp\x12\t.Actionsp\x1a\r.Transitionsp\"\x00\x12\x1f\n\x07Samplep\x12\x07.Emptyp\x1a\t.Actionsp\"\x00\x12\x1d\n\x06\x43losep\x12\x08.EnvIDsp\x1a\x07.Emptyp\"\x00\x62\x06proto3'
)




_NAMEP = _descriptor.Descriptor(
  name='Namep',
  full_name='Namep',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='Namep.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=14,
  serialized_end=35,
)


_INFOP = _descriptor.Descriptor(
  name='Infop',
  full_name='Infop',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='observation_space_type', full_name='Infop.observation_space_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observation_shape', full_name='Infop.observation_shape', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observation_space_low', full_name='Infop.observation_space_low', index=2,
      number=3, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observation_space_high', full_name='Infop.observation_space_high', index=3,
      number=4, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_actions', full_name='Infop.num_actions', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_episode_steps', full_name='Infop.max_episode_steps', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='action_space_type', full_name='Infop.action_space_type', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='env_id', full_name='Infop.env_id', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=38,
  serialized_end=258,
)


_EMPTYP = _descriptor.Descriptor(
  name='Emptyp',
  full_name='Emptyp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=260,
  serialized_end=268,
)


_OBSERVATIONP = _descriptor.Descriptor(
  name='Observationp',
  full_name='Observationp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='Observationp.data', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shape', full_name='Observationp.shape', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=270,
  serialized_end=313,
)


_OBSERVATIONSP = _descriptor.Descriptor(
  name='Observationsp',
  full_name='Observationsp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='obs', full_name='Observationsp.obs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=315,
  serialized_end=358,
)


_ACTIONSP = _descriptor.Descriptor(
  name='Actionsp',
  full_name='Actionsp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='env_id', full_name='Actionsp.env_id', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='Actionsp.data', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=360,
  serialized_end=400,
)


_ENVIDSP = _descriptor.Descriptor(
  name='EnvIDsp',
  full_name='EnvIDsp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='EnvIDsp.data', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=402,
  serialized_end=425,
)


_TRANSITIONP = _descriptor.Descriptor(
  name='Transitionp',
  full_name='Transitionp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='next_observation', full_name='Transitionp.next_observation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reward', full_name='Transitionp.reward', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='done', full_name='Transitionp.done', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=427,
  serialized_end=511,
)


_TRANSITIONSP = _descriptor.Descriptor(
  name='Transitionsp',
  full_name='Transitionsp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trans', full_name='Transitionsp.trans', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=513,
  serialized_end=556,
)


_RENDERMODESP = _descriptor.Descriptor(
  name='RenderModesp',
  full_name='RenderModesp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='env_id', full_name='RenderModesp.env_id', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mode', full_name='RenderModesp.mode', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=558,
  serialized_end=602,
)


_RENDEROUTSP = _descriptor.Descriptor(
  name='RenderOutsp',
  full_name='RenderOutsp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='rgb', full_name='RenderOutsp.rgb', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ansi', full_name='RenderOutsp.ansi', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=604,
  serialized_end=655,
)


_RGBARRYP = _descriptor.Descriptor(
  name='RGBArryp',
  full_name='RGBArryp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='RGBArryp.data', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=657,
  serialized_end=681,
)


_ENVSEEDSP = _descriptor.Descriptor(
  name='EnvSeedsp',
  full_name='EnvSeedsp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='env_id', full_name='EnvSeedsp.env_id', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='EnvSeedsp.data', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=683,
  serialized_end=724,
)

_OBSERVATIONSP.fields_by_name['obs'].message_type = _OBSERVATIONP
_TRANSITIONP.fields_by_name['next_observation'].message_type = _OBSERVATIONP
_TRANSITIONSP.fields_by_name['trans'].message_type = _TRANSITIONP
_RENDEROUTSP.fields_by_name['rgb'].message_type = _RGBARRYP
DESCRIPTOR.message_types_by_name['Namep'] = _NAMEP
DESCRIPTOR.message_types_by_name['Infop'] = _INFOP
DESCRIPTOR.message_types_by_name['Emptyp'] = _EMPTYP
DESCRIPTOR.message_types_by_name['Observationp'] = _OBSERVATIONP
DESCRIPTOR.message_types_by_name['Observationsp'] = _OBSERVATIONSP
DESCRIPTOR.message_types_by_name['Actionsp'] = _ACTIONSP
DESCRIPTOR.message_types_by_name['EnvIDsp'] = _ENVIDSP
DESCRIPTOR.message_types_by_name['Transitionp'] = _TRANSITIONP
DESCRIPTOR.message_types_by_name['Transitionsp'] = _TRANSITIONSP
DESCRIPTOR.message_types_by_name['RenderModesp'] = _RENDERMODESP
DESCRIPTOR.message_types_by_name['RenderOutsp'] = _RENDEROUTSP
DESCRIPTOR.message_types_by_name['RGBArryp'] = _RGBARRYP
DESCRIPTOR.message_types_by_name['EnvSeedsp'] = _ENVSEEDSP
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Namep = _reflection.GeneratedProtocolMessageType('Namep', (_message.Message,), {
  'DESCRIPTOR' : _NAMEP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:Namep)
  })
_sym_db.RegisterMessage(Namep)

Infop = _reflection.GeneratedProtocolMessageType('Infop', (_message.Message,), {
  'DESCRIPTOR' : _INFOP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:Infop)
  })
_sym_db.RegisterMessage(Infop)

Emptyp = _reflection.GeneratedProtocolMessageType('Emptyp', (_message.Message,), {
  'DESCRIPTOR' : _EMPTYP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:Emptyp)
  })
_sym_db.RegisterMessage(Emptyp)

Observationp = _reflection.GeneratedProtocolMessageType('Observationp', (_message.Message,), {
  'DESCRIPTOR' : _OBSERVATIONP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:Observationp)
  })
_sym_db.RegisterMessage(Observationp)

Observationsp = _reflection.GeneratedProtocolMessageType('Observationsp', (_message.Message,), {
  'DESCRIPTOR' : _OBSERVATIONSP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:Observationsp)
  })
_sym_db.RegisterMessage(Observationsp)

Actionsp = _reflection.GeneratedProtocolMessageType('Actionsp', (_message.Message,), {
  'DESCRIPTOR' : _ACTIONSP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:Actionsp)
  })
_sym_db.RegisterMessage(Actionsp)

EnvIDsp = _reflection.GeneratedProtocolMessageType('EnvIDsp', (_message.Message,), {
  'DESCRIPTOR' : _ENVIDSP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:EnvIDsp)
  })
_sym_db.RegisterMessage(EnvIDsp)

Transitionp = _reflection.GeneratedProtocolMessageType('Transitionp', (_message.Message,), {
  'DESCRIPTOR' : _TRANSITIONP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:Transitionp)
  })
_sym_db.RegisterMessage(Transitionp)

Transitionsp = _reflection.GeneratedProtocolMessageType('Transitionsp', (_message.Message,), {
  'DESCRIPTOR' : _TRANSITIONSP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:Transitionsp)
  })
_sym_db.RegisterMessage(Transitionsp)

RenderModesp = _reflection.GeneratedProtocolMessageType('RenderModesp', (_message.Message,), {
  'DESCRIPTOR' : _RENDERMODESP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:RenderModesp)
  })
_sym_db.RegisterMessage(RenderModesp)

RenderOutsp = _reflection.GeneratedProtocolMessageType('RenderOutsp', (_message.Message,), {
  'DESCRIPTOR' : _RENDEROUTSP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:RenderOutsp)
  })
_sym_db.RegisterMessage(RenderOutsp)

RGBArryp = _reflection.GeneratedProtocolMessageType('RGBArryp', (_message.Message,), {
  'DESCRIPTOR' : _RGBARRYP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:RGBArryp)
  })
_sym_db.RegisterMessage(RGBArryp)

EnvSeedsp = _reflection.GeneratedProtocolMessageType('EnvSeedsp', (_message.Message,), {
  'DESCRIPTOR' : _ENVSEEDSP,
  '__module__' : 'envp_pb2'
  # @@protoc_insertion_point(class_scope:EnvSeedsp)
  })
_sym_db.RegisterMessage(EnvSeedsp)



_ENVP = _descriptor.ServiceDescriptor(
  name='Envp',
  full_name='Envp',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=727,
  serialized_end=973,
  methods=[
  _descriptor.MethodDescriptor(
    name='Makep',
    full_name='Envp.Makep',
    index=0,
    containing_service=None,
    input_type=_NAMEP,
    output_type=_INFOP,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Resetp',
    full_name='Envp.Resetp',
    index=1,
    containing_service=None,
    input_type=_ENVIDSP,
    output_type=_OBSERVATIONSP,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Renderp',
    full_name='Envp.Renderp',
    index=2,
    containing_service=None,
    input_type=_RENDERMODESP,
    output_type=_RENDEROUTSP,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Seedp',
    full_name='Envp.Seedp',
    index=3,
    containing_service=None,
    input_type=_ENVSEEDSP,
    output_type=_EMPTYP,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Stepp',
    full_name='Envp.Stepp',
    index=4,
    containing_service=None,
    input_type=_ACTIONSP,
    output_type=_TRANSITIONSP,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Samplep',
    full_name='Envp.Samplep',
    index=5,
    containing_service=None,
    input_type=_EMPTYP,
    output_type=_ACTIONSP,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Closep',
    full_name='Envp.Closep',
    index=6,
    containing_service=None,
    input_type=_ENVIDSP,
    output_type=_EMPTYP,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_ENVP)

DESCRIPTOR.services_by_name['Envp'] = _ENVP

# @@protoc_insertion_point(module_scope)



import gym
from sasrl_env.common.python.env_pb2 import Info, Observation, Transition, Action, Empty, RenderOut
from sasrl_env.common.python.env_pb2_grpc import EnvServicer as Service

from sasrl_env.common.python.utils import get_logger
logger = get_logger(log_level='info')

def encode_observation(observation):
    return Observation(data=observation.ravel(), shape=observation.shape)

class Env(Service):

    def Make(self, name, _):
        name = name.data
        if not hasattr(self, 'env') or self.env.spec.id != name:
            self.env = gym.make(name)
        print('env {} created'.format(name))

        # check validity of observation_space
        try:
            self.env.observation_space
        except:
            raise NameError('Environment should have an observation_space object. Use either Box or Discrete to define the space object.')

        try:
            if self.env.observation_space.__class__ == gym.spaces.box.Box:
                observation_space_type = 'box'
            elif self.env.observation_space.__class__ == gym.spaces.Discrete:
                observation_space_type = 'discrete'
        except:
            raise NotImplementedError('Box and Discrete are the only supported observation_space types.')

        # check validity of action_space
        try:
            self.env.action_space
        except:
            raise NameError('Environment should have an action_space object of type Discrete.')

        if self.env.action_space.__class__ != gym.spaces.Discrete:
            action_space_type_ = "continues"
            num_actions = 0
            logger.warning('Discrete is the only supported space for action_space.')
        else:
            num_actions = self.env.action_space.n
            action_space_type_ = "discrete"

        return Info(observation_space_type=observation_space_type,
                    observation_shape=self.env.observation_space.shape,
                    num_actions=num_actions,
                    max_episode_steps=self.env._max_episode_steps,
                    action_space_type=action_space_type_)

    def Reset(self, empty, _):
        return encode_observation(self.env.reset())

    def Step(self, action, _):
        next_observation, reward, done, _ = self.env.step(action.data)
        next_observation = encode_observation(next_observation)
        return Transition(next_observation=next_observation,
                          reward=reward,
                          done=done)


    def Render(self, rendermode, _):
        res = self.env.render(rendermode.data)

        mode = rendermode.data
        if mode == 'rgb_array':
            reno = RenderOut(rgb_array=res.flatten())
        elif mode == 'ansi':
            reno = RenderOut(ansi=res)
        elif mode == 'human':
            reno = RenderOut()
        else:
            raise Exception("render mode {} not supported.".format(mode))

        return reno

    def Sample(self, empty, _):
        action = self.env.action_space.sample()
        return Action(data=action)

    def Close(self, empty, _):
        self.env.close()
        return Empty()

    def Seed(self, EnvSeed_, _):
        if hasattr(self.env, 'seed'):
            self.env.seed(EnvSeed_.data)
        else:
            logger.warning("There is no function to set seed in the environment.")
        return Empty()



from abc import ABC, abstractmethod
from enum import Enum


# code obtained from
# https://github.com/openai/baselines/blob/9cb7ece3387c4cb680fd831f38400b356bd599a6/baselines/common/vec_env/__init__.py

class EnvStatus(Enum):
    IDLE = 1
    WAITING = 2
    CLOSED = 3


class DictEnv(ABC):
    """
    Abstract class for asynchronous environments.
    """

    def __init__(self):
        self.num_envs = 0

    def inc_num(self, n=1):
        self.num_envs += n

    @abstractmethod
    def close(self, env_ids):
        """
        Clean up the environments' resources.
        """
        pass

    @abstractmethod
    def step(self, actions):
        """
        Do asynchoronous update in selected environments
        :param actions: a dict of environment ids with the actions in each
        Returns (obs, rews, dones, infos):
         - obs: an array of observations, or a tuple of
                arrays of observations.
         - rews: an array of rewards
         - dones: an array of "episode done" booleans
         - infos: a sequence of info objects
        """
        pass

    def render(self):
        print('Render not defined for %s' % self)

    @property
    def unwrapped(self):
        if isinstance(self, DictEnvWrapper):
            return self.denv.unwrapped
        else:
            return self


class DictEnvWrapper(DictEnv):
    def __init__(self, denv, observation_space=None, action_space=None):
        self.denv = denv
        DictEnv.__init__(self,
                         observation_space=observation_space or denv.observation_space,
                         action_space=action_space or denv.action_space)

    def step_async(self, actions):
        self.denv.step_async(actions)

    @abstractmethod
    def reset(self, env_ids):
        pass

    @abstractmethod
    def step_wait(self, actions):
        pass

    def close(self, env_ids):
        return self.denv.close(env_ids)

    def render(self):
        self.denv.render()


class CloudpickleWrapper(object):
    """
    Uses cloudpickle to serialize contents (otherwise multiprocessing tries to use pickle)
    """

    def __init__(self, x):
        self.x = x

    def __getstate__(self):
        import cloudpickle
        return cloudpickle.dumps(self.x)

    def __setstate__(self, ob):
        import pickle
        self.x = pickle.loads(ob)



# Copyright SAS Institute
#
#  Licensed under the Apache License, Version 2.0 (the License);
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

''' Install Remote Environment Using GRPC '''
import setuptools
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sasrl-env",
    version="0.0.2",
    author="SAS",
    author_email='support@sas.com',
    license='Apache 2.0',
    description="Generate protobuf files in python and C++ to be used for remote access to RL environment",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.sas.com/RezaNazari/rlenv_grpc.git",
    packages=setuptools.find_packages(),
    install_requires=['grpcio','protobuf','gym'],
    classifiers=["Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: env.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='env.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\tenv.proto\"\x14\n\x04Name\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\"\xcb\x01\n\x04Info\x12\x1e\n\x16observation_space_type\x18\x01 \x01(\t\x12\x19\n\x11observation_shape\x18\x02 \x03(\x05\x12\x1d\n\x15observation_space_low\x18\x03 \x03(\x02\x12\x1e\n\x16observation_space_high\x18\x04 \x03(\x02\x12\x13\n\x0bnum_actions\x18\x05 \x01(\x05\x12\x19\n\x11max_episode_steps\x18\x06 \x01(\x05\x12\x19\n\x11\x61\x63tion_space_type\x18\x07 \x01(\t\"\x07\n\x05\x45mpty\"*\n\x0bObservation\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x02\x12\r\n\x05shape\x18\x02 \x03(\x05\"\x16\n\x06\x41\x63tion\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x05\"R\n\nTransition\x12&\n\x10next_observation\x18\x01 \x01(\x0b\x32\x0c.Observation\x12\x0e\n\x06reward\x18\x02 \x01(\x02\x12\x0c\n\x04\x64one\x18\x03 \x01(\x08\"\x1a\n\nRenderMode\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\"&\n\tRenderOut\x12\x0b\n\x03rgb\x18\x01 \x03(\x05\x12\x0c\n\x04\x61nsi\x18\x02 \x01(\t\"\x17\n\x07\x45nvSeed\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x05\x32\xd7\x01\n\x03\x45nv\x12\x16\n\x04Make\x12\x05.Name\x1a\x05.Info\"\x00\x12\x1f\n\x05Reset\x12\x06.Empty\x1a\x0c.Observation\"\x00\x12\x1e\n\x04Step\x12\x07.Action\x1a\x0b.Transition\"\x00\x12#\n\x06Render\x12\x0b.RenderMode\x1a\n.RenderOut\"\x00\x12\x1a\n\x04Seed\x12\x08.EnvSeed\x1a\x06.Empty\"\x00\x12\x1b\n\x06Sample\x12\x06.Empty\x1a\x07.Action\"\x00\x12\x19\n\x05\x43lose\x12\x06.Empty\x1a\x06.Empty\"\x00\x62\x06proto3'
)




_NAME = _descriptor.Descriptor(
  name='Name',
  full_name='Name',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='Name.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=13,
  serialized_end=33,
)


_INFO = _descriptor.Descriptor(
  name='Info',
  full_name='Info',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='observation_space_type', full_name='Info.observation_space_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observation_shape', full_name='Info.observation_shape', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observation_space_low', full_name='Info.observation_space_low', index=2,
      number=3, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observation_space_high', full_name='Info.observation_space_high', index=3,
      number=4, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_actions', full_name='Info.num_actions', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_episode_steps', full_name='Info.max_episode_steps', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='action_space_type', full_name='Info.action_space_type', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=239,
)


_EMPTY = _descriptor.Descriptor(
  name='Empty',
  full_name='Empty',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=241,
  serialized_end=248,
)


_OBSERVATION = _descriptor.Descriptor(
  name='Observation',
  full_name='Observation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='Observation.data', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shape', full_name='Observation.shape', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=250,
  serialized_end=292,
)


_ACTION = _descriptor.Descriptor(
  name='Action',
  full_name='Action',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='Action.data', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=294,
  serialized_end=316,
)


_TRANSITION = _descriptor.Descriptor(
  name='Transition',
  full_name='Transition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='next_observation', full_name='Transition.next_observation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reward', full_name='Transition.reward', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='done', full_name='Transition.done', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=318,
  serialized_end=400,
)


_RENDERMODE = _descriptor.Descriptor(
  name='RenderMode',
  full_name='RenderMode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='RenderMode.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=402,
  serialized_end=428,
)


_RENDEROUT = _descriptor.Descriptor(
  name='RenderOut',
  full_name='RenderOut',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='rgb', full_name='RenderOut.rgb', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ansi', full_name='RenderOut.ansi', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=430,
  serialized_end=468,
)


_ENVSEED = _descriptor.Descriptor(
  name='EnvSeed',
  full_name='EnvSeed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='EnvSeed.data', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=470,
  serialized_end=493,
)

_TRANSITION.fields_by_name['next_observation'].message_type = _OBSERVATION
DESCRIPTOR.message_types_by_name['Name'] = _NAME
DESCRIPTOR.message_types_by_name['Info'] = _INFO
DESCRIPTOR.message_types_by_name['Empty'] = _EMPTY
DESCRIPTOR.message_types_by_name['Observation'] = _OBSERVATION
DESCRIPTOR.message_types_by_name['Action'] = _ACTION
DESCRIPTOR.message_types_by_name['Transition'] = _TRANSITION
DESCRIPTOR.message_types_by_name['RenderMode'] = _RENDERMODE
DESCRIPTOR.message_types_by_name['RenderOut'] = _RENDEROUT
DESCRIPTOR.message_types_by_name['EnvSeed'] = _ENVSEED
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Name = _reflection.GeneratedProtocolMessageType('Name', (_message.Message,), {
  'DESCRIPTOR' : _NAME,
  '__module__' : 'env_pb2'
  # @@protoc_insertion_point(class_scope:Name)
  })
_sym_db.RegisterMessage(Name)

Info = _reflection.GeneratedProtocolMessageType('Info', (_message.Message,), {
  'DESCRIPTOR' : _INFO,
  '__module__' : 'env_pb2'
  # @@protoc_insertion_point(class_scope:Info)
  })
_sym_db.RegisterMessage(Info)

Empty = _reflection.GeneratedProtocolMessageType('Empty', (_message.Message,), {
  'DESCRIPTOR' : _EMPTY,
  '__module__' : 'env_pb2'
  # @@protoc_insertion_point(class_scope:Empty)
  })
_sym_db.RegisterMessage(Empty)

Observation = _reflection.GeneratedProtocolMessageType('Observation', (_message.Message,), {
  'DESCRIPTOR' : _OBSERVATION,
  '__module__' : 'env_pb2'
  # @@protoc_insertion_point(class_scope:Observation)
  })
_sym_db.RegisterMessage(Observation)

Action = _reflection.GeneratedProtocolMessageType('Action', (_message.Message,), {
  'DESCRIPTOR' : _ACTION,
  '__module__' : 'env_pb2'
  # @@protoc_insertion_point(class_scope:Action)
  })
_sym_db.RegisterMessage(Action)

Transition = _reflection.GeneratedProtocolMessageType('Transition', (_message.Message,), {
  'DESCRIPTOR' : _TRANSITION,
  '__module__' : 'env_pb2'
  # @@protoc_insertion_point(class_scope:Transition)
  })
_sym_db.RegisterMessage(Transition)

RenderMode = _reflection.GeneratedProtocolMessageType('RenderMode', (_message.Message,), {
  'DESCRIPTOR' : _RENDERMODE,
  '__module__' : 'env_pb2'
  # @@protoc_insertion_point(class_scope:RenderMode)
  })
_sym_db.RegisterMessage(RenderMode)

RenderOut = _reflection.GeneratedProtocolMessageType('RenderOut', (_message.Message,), {
  'DESCRIPTOR' : _RENDEROUT,
  '__module__' : 'env_pb2'
  # @@protoc_insertion_point(class_scope:RenderOut)
  })
_sym_db.RegisterMessage(RenderOut)

EnvSeed = _reflection.GeneratedProtocolMessageType('EnvSeed', (_message.Message,), {
  'DESCRIPTOR' : _ENVSEED,
  '__module__' : 'env_pb2'
  # @@protoc_insertion_point(class_scope:EnvSeed)
  })
_sym_db.RegisterMessage(EnvSeed)



_ENV = _descriptor.ServiceDescriptor(
  name='Env',
  full_name='Env',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=496,
  serialized_end=711,
  methods=[
  _descriptor.MethodDescriptor(
    name='Make',
    full_name='Env.Make',
    index=0,
    containing_service=None,
    input_type=_NAME,
    output_type=_INFO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Reset',
    full_name='Env.Reset',
    index=1,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_OBSERVATION,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Step',
    full_name='Env.Step',
    index=2,
    containing_service=None,
    input_type=_ACTION,
    output_type=_TRANSITION,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Render',
    full_name='Env.Render',
    index=3,
    containing_service=None,
    input_type=_RENDERMODE,
    output_type=_RENDEROUT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Seed',
    full_name='Env.Seed',
    index=4,
    containing_service=None,
    input_type=_ENVSEED,
    output_type=_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Sample',
    full_name='Env.Sample',
    index=5,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_ACTION,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Close',
    full_name='Env.Close',
    index=6,
    containing_service=None,
    input_type=_EMPTY,
    output_type=_EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_ENV)

DESCRIPTOR.services_by_name['Env'] = _ENV

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

# import env_pb2 as env__pb2
import sasrl_env.common.python.env_pb2 as env__pb2


class EnvStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Make = channel.unary_unary(
                '/Env/Make',
                request_serializer=env__pb2.Name.SerializeToString,
                response_deserializer=env__pb2.Info.FromString,
                )
        self.Reset = channel.unary_unary(
                '/Env/Reset',
                request_serializer=env__pb2.Empty.SerializeToString,
                response_deserializer=env__pb2.Observation.FromString,
                )
        self.Step = channel.unary_unary(
                '/Env/Step',
                request_serializer=env__pb2.Action.SerializeToString,
                response_deserializer=env__pb2.Transition.FromString,
                )
        self.Render = channel.unary_unary(
                '/Env/Render',
                request_serializer=env__pb2.RenderMode.SerializeToString,
                response_deserializer=env__pb2.RenderOut.FromString,
                )
        self.Seed = channel.unary_unary(
                '/Env/Seed',
                request_serializer=env__pb2.EnvSeed.SerializeToString,
                response_deserializer=env__pb2.Empty.FromString,
                )
        self.Sample = channel.unary_unary(
                '/Env/Sample',
                request_serializer=env__pb2.Empty.SerializeToString,
                response_deserializer=env__pb2.Action.FromString,
                )
        self.Close = channel.unary_unary(
                '/Env/Close',
                request_serializer=env__pb2.Empty.SerializeToString,
                response_deserializer=env__pb2.Empty.FromString,
                )


class EnvServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Make(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Reset(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Step(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Render(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Seed(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Sample(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Close(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_EnvServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Make': grpc.unary_unary_rpc_method_handler(
                    servicer.Make,
                    request_deserializer=env__pb2.Name.FromString,
                    response_serializer=env__pb2.Info.SerializeToString,
            ),
            'Reset': grpc.unary_unary_rpc_method_handler(
                    servicer.Reset,
                    request_deserializer=env__pb2.Empty.FromString,
                    response_serializer=env__pb2.Observation.SerializeToString,
            ),
            'Step': grpc.unary_unary_rpc_method_handler(
                    servicer.Step,
                    request_deserializer=env__pb2.Action.FromString,
                    response_serializer=env__pb2.Transition.SerializeToString,
            ),
            'Render': grpc.unary_unary_rpc_method_handler(
                    servicer.Render,
                    request_deserializer=env__pb2.RenderMode.FromString,
                    response_serializer=env__pb2.RenderOut.SerializeToString,
            ),
            'Seed': grpc.unary_unary_rpc_method_handler(
                    servicer.Seed,
                    request_deserializer=env__pb2.EnvSeed.FromString,
                    response_serializer=env__pb2.Empty.SerializeToString,
            ),
            'Sample': grpc.unary_unary_rpc_method_handler(
                    servicer.Sample,
                    request_deserializer=env__pb2.Empty.FromString,
                    response_serializer=env__pb2.Action.SerializeToString,
            ),
            'Close': grpc.unary_unary_rpc_method_handler(
                    servicer.Close,
                    request_deserializer=env__pb2.Empty.FromString,
                    response_serializer=env__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Env', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Env(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Make(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Env/Make',
            env__pb2.Name.SerializeToString,
            env__pb2.Info.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Reset(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Env/Reset',
            env__pb2.Empty.SerializeToString,
            env__pb2.Observation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Step(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Env/Step',
            env__pb2.Action.SerializeToString,
            env__pb2.Transition.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Render(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Env/Render',
            env__pb2.RenderMode.SerializeToString,
            env__pb2.RenderOut.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Seed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Env/Seed',
            env__pb2.EnvSeed.SerializeToString,
            env__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Sample(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Env/Sample',
            env__pb2.Empty.SerializeToString,
            env__pb2.Action.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Close(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Env/Close',
            env__pb2.Empty.SerializeToString,
            env__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
